from openai import OpenAI
from termcolor import colored
import os
import json

class MainMemory:
    def __init__(self):
        self.MAX_WORDS_IN_MEMORY = 10000
        self.messages = [
            {"role": "system", "content": "You are a helpful assistant."},
        ]

    def add_message(self, role, content):
        self.messages.append({"role": role, "content": content})

    def get_messages(self):
        return self.messages
    
    def trim_memory(self):
        # count the words in the memory
        while len(" ".join([message["content"] for message in self.messages]).split()) > self.MAX_WORDS_IN_MEMORY:
            # we pop the second element because the first one is the system message
            self.messages.pop(1)

    # we save the message to file for observation purposes
    def save_messages_to_File(self):
        with open('messages.json', 'w') as outfile:
            json.dump(self.messages, outfile, indent=4)

class ChatBot:
    def __init__(self, api_key="YOUR_KEY_HERE"):
        self.client = OpenAI(api_key=os.getenv("OPENAI_API_KEY") or api_key)
        self.memory = MainMemory()

    def chat_main(self, user_input):
        self.memory.add_message("user", user_input)

        try:
            response = self.client.chat.completions.create(
                model="gpt-4-1106-preview",
                stream=True,
                messages=self.memory.get_messages(),
            )

            responses = ''
            for chunk in response:
                if chunk.choices[0].delta.content: 
                    text_chunk = chunk.choices[0].delta.content 
                    print(colored(text_chunk, 'green'), end="", flush=True)
                    responses += str(text_chunk)
            print("\n")

            self.memory.add_message("assistant", responses)
            self.memory.trim_memory()
            self.memory.save_messages_to_File()
        except Exception as e:
            print(f"OpenAI API Error: {e}")

        return responses